==> veil-fabric-1.21-1.0.0.12-sources.jar: foundry/veil/fabric/FabricQuasarParticleHandler.java <==
package foundry.veil.fabric;

import foundry.veil.api.client.render.CachedBufferSource;
import foundry.veil.api.client.render.VeilRenderSystem;
import foundry.veil.api.client.render.VeilRenderer;
import foundry.veil.api.event.VeilRenderLevelStageEvent;
import foundry.veil.fabric.event.FabricFreeNativeResourcesEvent;
import foundry.veil.fabric.event.FabricVeilRenderLevelStageEvent;
import net.minecraft.class_638;
import org.jetbrains.annotations.ApiStatus;

@ApiStatus.Internal
public class FabricQuasarParticleHandler {

    private static CachedBufferSource cachedBufferSource;

    private static void free() {
        if (cachedBufferSource != null) {
            cachedBufferSource.free();
            cachedBufferSource = null;
        }
    }

    public static void setLevel(class_638 level) {
        VeilRenderSystem.renderer().getParticleManager().setLevel(level);
        free();
    }

    public static void init() {
        FabricFreeNativeResourcesEvent.EVENT.register(FabricQuasarParticleHandler::free);
        FabricVeilRenderLevelStageEvent.EVENT.register((stage, levelRenderer, bufferSource, poseStack, modelMatrix, projectionMatrix, renderTick, deltaTracker, camera, frustum) -> {
            if (stage == VeilRenderLevelStageEvent.Stage.AFTER_PARTICLES) {
                if (cachedBufferSource == null) {
                    cachedBufferSource = new CachedBufferSource();
                }
                VeilRenderSystem.renderer().getParticleManager().render(poseStack, cachedBufferSource, camera, VeilRenderer.getCullingFrustum(), deltaTracker.method_60638());
                cachedBufferSource.endBatch();
            }
        });
    }
}

==> veil-fabric-1.21-1.0.0.22-sources.jar: foundry/veil/fabric/FabricQuasarParticleHandler.java <==
package foundry.veil.fabric;

import foundry.veil.api.client.render.CachedBufferSource;
import foundry.veil.api.client.render.VeilRenderSystem;
import foundry.veil.api.client.render.VeilRenderer;
import foundry.veil.api.event.VeilRenderLevelStageEvent;
import foundry.veil.fabric.event.FabricFreeNativeResourcesEvent;
import foundry.veil.fabric.event.FabricVeilRenderLevelStageEvent;
import net.minecraft.class_638;
import org.jetbrains.annotations.ApiStatus;

@ApiStatus.Internal
public class FabricQuasarParticleHandler {

    private static CachedBufferSource cachedBufferSource;

    private static void free() {
        if (cachedBufferSource != null) {
            cachedBufferSource.free();
            cachedBufferSource = null;
        }
    }

    public static void setLevel(class_638 level) {
        VeilRenderSystem.renderer().getParticleManager().setLevel(level);
        free();
    }

    public static void init() {
        FabricFreeNativeResourcesEvent.EVENT.register(FabricQuasarParticleHandler::free);
        FabricVeilRenderLevelStageEvent.EVENT.register((stage, levelRenderer, bufferSource, poseStack, modelMatrix, projectionMatrix, renderTick, deltaTracker, camera, frustum) -> {
            if (stage == VeilRenderLevelStageEvent.Stage.AFTER_PARTICLES) {
                if (cachedBufferSource == null) {
                    cachedBufferSource = new CachedBufferSource();
                }
                VeilRenderSystem.renderer().getParticleManager().render(poseStack, cachedBufferSource, camera, VeilRenderer.getCullingFrustum(), deltaTracker.method_60637(false));
                cachedBufferSource.endBatch();
            }
        });
    }
}
